package commun;

/**
 * <code>Piece</code> est l'interface utilisée pour
 * représenter le contenu  des cases. Contient une méthode
 * qui renvoi un caractère qui représente la pièce (pour affichage textuel).
 * 
 */
public interface Piece {
	 
    /**
     * <code>getName</code> retourne le nom de la pièce (1 caractère:
     * par exemple o et x pour les pions blancs et noirs, O et X pour
     * les dames blanches et noires).  
     * 
     * @return le caractère représentant la pièce.
     */
    public Character getName();

   
     /** <code>getPosition</code> retourne la position courante de la pièce
     * (<code>this</code>) sur son plateau.
     * 
     * @return une <code>Coordonnees</code> correspondant à la position.
     */
    public Coordonnees getPosition();

    /** <code>peutSePlacerA(c)</code> retourne <code>true</code> si la pièce
     * (supposée pas encore posée sur le plateau) peut se placer à la
     * coordonnée <code>c</code>. Retourne <code>false</code> sinon.
     * 
     * @return un booléen.
     */
    public boolean peutSePlacerA(Coordonnees c);
    
    /** <code>estGagnant()</code> retourne <code>true</code> si la pièce
     * provoque la victoire du joueur qui la possède. Retourne
     * <code>false</code> sinon.
     * 
     * @return un booléen.
     */
    public boolean estGagnant();
    
    /** <code>estVide()</code> retourne <code>true</code> si la pièce est vide.
     *  Retourne <code>false</code> sinon.
     * 
     * @return un booléen.
     */
    public boolean estVide();

    /** <code>place(c)</code> pose la pièce sur son plateau à la coordonnée
     * <code>c/code>.
     */
    public void place(Coordonnees c);    

    /** <code>p.supprime</code> enlève la pièce {@code p} du plateau
     * {@code iop}. 
     * @throws NonImplante pour les jeux sans prise et/ou sans mouvements.*/
    public void supprime();   
    
    /** <code>sameName(p)</code> retourne <code>true</code> si this et p ont le même nom.
     *  Retourne <code>false</code> sinon.
     * 
     * @return un booléen.
     */
    public boolean sameName(Piece p);
 
}
