package commun;

import java.io.*;

/**
 * La classe <code>Terminal</code> sert aux étudiants pour leurs
entrée/sorties textuelles dans le terminal. C'est un ensemble de
méthodes statiques permettant de lire et écrire les types suivants:
<code>String</code>, <code>int</code>, <code>bool</code>,
<code>double</code> et <code>char</code>. <p>

Par exemple si on lance le code suivant:<p>

<code>
int i1 = Terminal.lireInt();<br>
int i2 = Terminal.lireInt();<br>
Terminal.printInt(s1+s2);
</code><p>

Et que l'utilisateur entre les caractères suivants (ou les
copie/colle):<p>

<code>
23[entrée]<br>32[entrée]
</code><p>

les deux entier <code>i1</code>(23) et <code>i2</code>(32) sont lus
et le programme affiche:<p>

<code>66</code><p>

Remarque: Les entrées sont bufferisées. C'est-à-dire que chaque
méthode ne consomme que les caractères dont elle a besoin, dans
l'ordre où ils ont été entrés et sans en "sauter". Les caractères
non consommés sont prêts à être consommés par les prochains appels
aux méthodes de la classe <code>Terminal</code>.



@author François Barthélémy
@version 1.0
 */
public class Terminal {

    static BufferedReader in =
            new BufferedReader(new InputStreamReader(System.in));

    /** Pour lire un <code>String</code>.
    @return la chaîne tapée jusqu'au premier retour à la ligne
    (<code>entrée</code>).
    
    @throws TerminalException
     */
    public static String lireString() {
        String tmp = "";
        char C = '\0';
        try {
            tmp = in.readLine();
        } catch (IOException e) {
            exceptionHandler(e);
        }
        return tmp;
    } // fin de lireString()

    /** Pour lire un <code>int</code>
    @return l'entier correspondant à la chaîne tapée. Retourne
    <code>0</code> si pas d'entier reconnaissable.
    @throws TerminalException */
    public static int lireInt() {
        int x = 0;
        try {
            x = Integer.parseInt(lireString());
        } catch (NumberFormatException e) {
            exceptionHandler(e);
        }
        return x;
    }

    /** Pour lire un <code>boolean</code>
    @return le booléen correspondant à la chaîne tapée. Retourne
    <code>true</code> si pas de booléen reconnaissable.
    @throws TerminalException */
    public static boolean lireBoolean() // Lire un entier
    {
        boolean b = true;
        try {
            b = Boolean.valueOf(lireString()).booleanValue();
        } catch (NumberFormatException e) {
            exceptionHandler(e);
        }
        return b;
    }

    /** Pour lire un <code>double</code>
    @return le double correspondant à la chaîne tapée. Retourne
    <code>0.0</code> si pas de double reconnaissable.
    @throws TerminalException */
    public static double lireDouble() // Lire un double
    {
        double x = 0.0;
        try {
            x = Double.valueOf(lireString()).doubleValue();
        } catch (NumberFormatException e) {
            exceptionHandler(e);
        }
        return x;
    }

    /** Pour lire un <code>char</code>
    @return le caract�re tap�. */
    public static char lireChar() // Lire un caractere
    {
        String tmp = lireString();
        if (tmp.length() == 0) {
            return '\n';
        } else {
            return tmp.charAt(0);
        }
    }

    /** Pour écrire une chaîne de caractères dans le terminal.
    @throws TerminalException */
    public static void ecrireString(String s) { // Afficher un String
        try {
            System.out.print(s);
        } catch (Exception ex) {
            exceptionHandler(ex);
        }
    }

    /** Pour écrire une chaîne de caractères dans le terminal et
    ajouter un retour à la ligne.
    @throws TerminalException */
    public static void ecrireStringln(String s) // Afficher un String
    {
        ecrireString(s);
        sautDeLigne();
    } // fin de ecrireStringln()

    /** Pour écrire un entier dans le terminal.
    @throws TerminalException */
    public static void ecrireInt(int i) // Afficher un entier
    {
        ecrireString("" + i);
    }

    /** Pour écrire un entier dans le terminal et ajouter un retour à
    la ligne.
    @throws TerminalException */
    public static void ecrireIntln(int i) // Afficher un entier
    {
        ecrireString("" + i);
        sautDeLigne();
    }

    /** Pour écrire un booléen dans le terminal.
    @throws TerminalException */
    public static void ecrireBoolean(boolean b) {
        ecrireString("" + b);
    }

    /** Pour écrire un bool�en dans le terminal et ajouter un retour
    à la ligne.
    @throws TerminalException */
    public static void ecrireBooleanln(boolean b) {
        ecrireString("" + b);
        sautDeLigne();
    }

    /** Pour écrire un double dans le terminal.
    @throws TerminalException */
    public static void ecrireDouble(double d) // Afficher un double
    {
        ecrireString("" + d);
    }

    /** Pour écrire un double dans le terminal et ajouter un retour à
    la ligne. 
    @throws TerminalException */
    public static void ecrireDoubleln(double d) // Afficher un double
    {
        ecrireDouble(d);
        sautDeLigne();
    }

    /** Pour écrire un caractère dans le terminal.
    @throws TerminalException */
    public static void ecrireChar(char c) // Afficher un caractere
    {
        ecrireString("" + c);
    }

    /** Pour écrire un caractère dans le terminal et ajouter un retour
    à la ligne.
    @throws TerminalException */
    public static void ecrireCharln(char c) // Afficher un caractere
    {
        ecrireChar(c);
        sautDeLigne();
    }

    /** Pour �crire un retour à la ligne dans le terminal.
    @throws TerminalException */
    public static void sautDeLigne() {
        try {
            System.out.println();
        } catch (Exception ex) {
            exceptionHandler(ex);
        }
    }

    /** Fonction auxiliaire, ne pas utiliser. */
    protected static void exceptionHandler(Exception ex) {
        TerminalException err = new TerminalException(ex);
        throw err;
    }

    /** Pour afficher une exception dans le terminal. */
    public static void ecrireException(Throwable ex) {
        ecrireString(ex.toString());
        ex.printStackTrace(System.err);
    }
}

/** Exception levée par les fonctions de la classe #Terminal */
class TerminalException extends RuntimeException {

    Exception ex;

    TerminalException(Exception e) {
        ex = e;
    }
}
