
package morpion;
import java.util.ArrayList;

import commun.*;
/*
 * Cette classe implante les fonctionnalités communes à toutes les pièces,
 * indépendamment du jeu concerné.
 */

/**
 *
 * @author courtieu, aponte
 */
public abstract class PieceAbstraite implements Piece {

    protected Coordonnees position; // position dans son plateau
    protected Plateau tableau;      // Une pièce connait son plateau
    protected Character charname;   // Caractère pour afficher la pièce.


    /** Associe un caractère, un plateau et une position à la pièce.
     * @param c
     * @param t
     * @param col
     * @param ch 
     */
    public PieceAbstraite(Coordonnees c, Plateau t,  Character ch) {
        this.position = c;
        this.tableau = t;
        this.charname = ch;
        t.setPiece(c, this);
    }

    /** Associe un plateau et une position à la pièce. */
    public PieceAbstraite(int x, int y, Plateau t, Character ch) {
        this.position = new Coordonnees(x, y);
        this.tableau = t;
        this.charname = ch;
        t.setPiece(x, y, this);
    }
 /**
  * Création d'une pièce non encore positionée sur le plateau
  */
    public PieceAbstraite(Plateau t, Character ch) {
        this.tableau = t;
        this.charname = ch;
    }

    /** <code>getPosition()</code> retourne la position de la pièce sur
     * le plateau. */
    public Coordonnees getPosition() {
        return this.position;
    }

    public Plateau getPlateau() {
        return this.tableau;
    }

    public Character getName(){
        return this.charname;
    }

    /** <code>place(c)</code> place la pièce (this) dans la coordonnée c,
     * sans aucune vérification. */
    public void place(Coordonnees c) {
        this.tableau.setPiece(c, this);
        this.position = c;
    }

    public void supprime(){
        this.tableau.setPiece(this.getPosition(),
                              new PieceVide(this.getPosition(),this.tableau));
        this.tableau = null;
        this.position = null;
    }
    
    public boolean estVide() {
        return false;
    }

    public boolean peutSePlacerA(Coordonnees c){
        return this.getPlateau().getPiece(c).estVide();
    }
    
    public boolean sameName(Piece p){
	    	return ((char) p.getName() == (char) this.getName());
    }
    
    @Override
	/**
	 * Vérifie tous les alignements où se trouve la pièce.
	 */
	public boolean estGagnant() {
		Plateau pt= this.getPlateau();
		Coordonnees pos = this.getPosition();
		int dg = pos.distBordG(pt);
		int dd = pos.distBordD(pt);
		ArrayList<Coordonnees> horizontal= pos.horizontal(dg, dd);
		for (Coordonnees c: horizontal ){
			if (!pt.getPiece(c).sameName(this))
				return false;
		}
		return true; // incomplet: vérifier alignement vertical et éventuellement diagonal
	}
    
       
}
